<?php

namespace App\Commands;

use App\Commands\Concerns\ClonesProjects;
use App\Commands\Concerns\SetProjectStorage;
use App\Git\ResetGitTask;
use App\Installation\InstructionManual;
use App\Projects\ProjectBases;
use App\Projects\ProjectName;
use App\Tasks\CopyProjectTask;
use App\Tasks\InitialiseProject;
use LaravelZero\Framework\Commands\Command;

class NewProject extends Command
{
    use ClonesProjects, SetProjectStorage;

    /**
     * The signature of the command.
     *
     * @var string
     */
    protected $signature = 'new {name : Name of the application you want to create (required)} 
                    {--type= : The type of application to install (optional)} 
                    {-f|--folder= : The relative folder to create the application in (optional)}';

    /**
     * The description of the command.
     *
     * @var string
     */
    protected $description = 'Create a new an application';

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $type = $this->decideProjectType();
        $projectBase = ProjectBases::findByType($type);
        $projectName = $this->argument('name');
        $destinationFolder = $this->option('folder') ?? $projectName;
        ProjectName::save($projectName);

        $this->setProcessTitle('Creating initial project');
        $this->cloneProject($projectBase);
        $this->task('Creating your new project', new CopyProjectTask($projectBase, $destinationFolder));
        $this->createProjectDisk($destinationFolder);

        $this->task('Resetting the git history', new ResetGitTask);

        InitialiseProject::run($projectBase->label, $this, $this->app[InstructionManual::class]);
        ProjectName::forget();
    }

    private function decideProjectType(): string
    {
        $type = (string) $this->option('type');

        if (blank($type)) {
            $type = $this->menu(
                'What type of project do you want to create?',
                ProjectBases::all()->pluck('label', 'type')->toArray()
            )->disableDefaultItems()->open();
        }

        return $type;
    }
}
