<?php

namespace App\Git;

use App\Oauth2\GitlabOptionsProvider;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Token\AccessTokenInterface;
use Omines\OAuth2\Client\Provider\Gitlab as GitlabProvider;

class AccessToken
{
    public static function retrieve(callable $credentialsFetcher): ?string
    {
        $accessToken = cache()->get('gitlab-access-token');

        if ($accessToken === null) {
            $accessToken = self::requestAccessToken(...$credentialsFetcher());
        }

        cache()->forever('gitlab-access-token', $accessToken);

        return $accessToken;
    }

    private static function requestAccessToken(string $username, string $password): ?AccessTokenInterface
    {
        $provider = new GitlabProvider([
            'clientId' => 'the_id',
            'clientSecret' => 'the_secret',
            'redirectUri' => '',
            'domain' => 'https://git.drp.digital',
        ]);
        $provider->setOptionProvider(new GitlabOptionsProvider);

        try {
            return $provider->getAccessToken('password', [
                'username' => $username,
                'password' => $password,
            ]);
        } catch (IdentityProviderException $e) {
            return null;
        }
    }
}
