<?php

namespace App\Installation;

use App\Database\Env\Writer;
use App\Installation\Commands\ComposerInstall;
use App\Installation\Commands\CreateDatabase;
use App\Installation\Commands\NpmBuild;
use App\Installation\Commands\NpmInstall;
use App\Installation\Contracts\Instructions;
use App\Installation\Contracts\RequiresDatabase;
use Illuminate\Support\Facades\Storage;

class WordpressInstructions extends Instructions implements RequiresDatabase
{
    public function name(): string
    {
        return 'Wordpress';
    }

    public function canRun(): bool
    {
        return Storage::disk('project')->exists('wp-config.php');
    }

    protected function commands(): array
    {
        return array_filter([
            $this->command('cp .env.example .env'),
            new ComposerInstall,
            new CreateDatabase(new Writer([
                'dbHost' => 'DB_HOST',
                'dbPort' => 'DB_PORT',
                'dbUsername' => 'DB_USER',
                'dbPassword' => 'DB_PASSWORD',
                'dbName' => 'DB_NAME',
            ])),
            new NpmInstall,
            new NpmBuild,
        ]);
    }
}
