<?php

namespace App\Tasks;

use App\Git\Cloneable;
use ErrorException;
use Illuminate\Support\Facades\Storage;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Finder\SplFileInfo;

class ApplySymlinks
{
    /**
     * @var \App\Git\Cloneable
     */
    private $cloneable;

    /**
     * @var string
     */
    private $destination;

    public function __construct(Cloneable $cloneable, string $destination)
    {
        $this->cloneable = $cloneable;
        $this->destination = $destination;
    }

    public function __invoke(): void
    {
        $finder = (new Finder)->in(Storage::disk('git-clone')->path($this->cloneable->getFolder()))->files();
        collect($finder)->filter(function (SplFileInfo $file) {
            return $file->isLink();
        })->each(function (SplFileInfo $file) {
            $directory = Storage::disk('local')->path($this->destination . '/' . $file->getRelativePath());

            $this->symlink($file->getFilename(), $directory, $file->getLinkTarget());
        });
    }

    private function symlink(string $file, string $directory, string $linkTarget): void
    {
        $original = getcwd();
        chdir($directory);
        try {
            symlink($linkTarget, $file);
        } catch (ErrorException $exception) {
        }
        chdir($original);
    }
}
